function [eq,HLI,low,med,high,rem] = equilib(D, writeout)
%EQUILIB Calculate shoe equilibrium.
%   equilib(D,writeout) calculates the equilibrium distribution on the state
%   of a shoe with D decks (multiples of 1/4) in it according to the Markov
%   chain strategy outlined by Wakin and Rozell (2004).  When writeout==1,
%   the results of this calculation as well as auxilliary information about
%   each state is written out to a MAT file.  When writeout==2, only the
%   equilibrium information is written out to the file (default
%   writeout==1). Please be warned that as the number of decks increases,
%   this computation can take a long time and resulting file sizes can be
%   large ~6MB for D=4, writeout==1).
%   
%   If a file is written, it is named with eq<num decks>.mat where <num
%   decks> is the number of decks in the shoe (fractions represented such
%   that 1/4 is given the string 0_25).  The file has the following vectors
%   of information:
%
%   low -  the number of low cards played in each state
%   med -  the number of medium cards played in each state
%   high - the number of high cards played in each state
%   rem -  the number of cards remaining in the shoe in each state
%   HLI -  the HLI number in each state
%   eq -   the equilibrium of each state
%   
%   If there are D decks in the shoe, then the number of states in the
%   Markov chain is equal to the number of combinations of low, medium and
%   high cards that can be played.  In our algorithm for computing the
%   equilibrium over these state, each state (L,M,H) must be given a unique
%   identifying number to use as an index into a vector.  We use Matlab's
%   built in index-to-subscript converters to do this.  
%   
%   For example, to convert from an index c to the triple (L,M,H) we use:
%   [L,M,H]=ind2sub([Ltot+1, Mtot+1, Htot+1], c);
%   L=L-1;
%   M=M-1;
%   H=H-1;  
%   
%   To convert from a triple (L,M,H) to an index c we use:
%   c = sub2ind([Ltot+1, Mtot+1, Htot+1], L+1, M+1, H+1)  
  
%   NOTE: Still not sure why there is probability assigned to last state,
%   since that is all zeros. Is this a bug?

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Blackjack Markov Chain Toolbox, v1.0 (08 Jan 2009)
%%% 
%%% Companion to manuscript 
%%%    "A Markov Chain Analysis of Blackjack Strategy"
%%%    by Michael B. Wakin and Christopher J. Rozell, 2004.
%%%    available at: http://inside.mines.edu/~mwakin/papers/mcbj.pdf
%%%
%%% Software Copyright 2009 by 
%%%    Michael B. Wakin, Colorado School of Mines, mwakin@mines.edu
%%%    Christopher J. Rozell, Georgia Tech, crozell@gatech.edu 
%%%
%%% This software is released under a Creative Commons license
%%% (Attribution 3.0 Unported): 
%%%     http://creativecommons.org/licenses/by/3.0/
%%% You are free:
%%%    * to Share  to copy, distribute and transmit the work
%%%    * to Remix  to adapt the work
%%% Under the following conditions:
%%%   * You must attribute the work by retaining this header information
%%%     crediting the original authors and linking to the original
%%%     manuscript.
%%%   * For any reuse or distribution, you must make clear to others the 
%%%     license terms of this work.
%%%   * Any of the above conditions can be waived if you get permission 
%%%     from the copyright holder.
%%%   * Nothing in this license impairs or restricts the author's 
%%%     moral rights.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


if(nargin<2)
  writeout=1;
end
  
%the number of decks in the shoe must be a multiple of 1/4 (for integer
%numbers of cards)
if(mod(D,1/4)~=0)
  error('D can only be in multiples of 1/4');
end
  

cards = 1:52*D;         %vector of card numbers
N=max(cards);           %total number of cards

Ltot = 20*D;        %total low cards
Mtot = 12*D;        %total med cards
Htot = 20*D;        %total high cards


numstates = ((Ltot+1)*(Mtot+1)*(Htot+1));  %total number of possible
                                           %states in the Markov Chain,
                                           %representing all possible
                                           %combinations of low, medium
                                           %and high card counts

					   
HLI  =  zeros(1, numstates);    %High-Low index calculated for each state
low  =  zeros(1, numstates);    %'low' card count for each state 
med  =  zeros(1, numstates);    %'medium' card count for each state 
high =  zeros(1, numstates);    %'high' card count for each state 
rem  =  zeros(1, numstates);    %number of cards unplayed cards in each state

%call reshuffle to calculate the reshuffle transition probabilites for
%each state (depending on the number of cards played)
[resh_trans_pr, card_eq]=reshuffle(D);

eq = zeros(1, numstates);       %vector to hold the equilibrium
                                %probabilites for each state

%Each state has a unique decimal number.  This 	number is mapped uniquely
%to an ordered triple representing the number of low, medium, and high
%cards that have been played in that state, which are recorded in the
%voctor 'sates'.  The following loop examines each state and calculates 
%the necessary 'statistics' for that state.
states = zeros(3,numstates);
for c=1:numstates
  [low(c),med(c),high(c)]=ind2sub([Ltot+1, Mtot+1, Htot+1], c);
  low(c) = low(c)-1;
  med(c) = med(c)-1;
  high(c) = high(c)-1;
  states(:,c)=[low(c) med(c) high(c)]';
  rem(c) = N-(low(c)+med(c)+high(c));
  HLI(c) = ((low(c)-high(c))/rem(c))*100;
end

state_cards = sum(states);  %for each state, record the total number of
                            %cards played

			    
%Each state conceptually belongs to one column representing the total
%number of cards played.  The equilibrium for each of these 'columns'
%depends on the reshuffle scheme, and has been calculated in the call to
%the 'reshuffle' function.  Now we must determine which states belong to
%each column, and distribute the columnar equilibrium appropriately among
%those states.
			    
%since there is only one card in first column, equilibruim is same as the
%columnar equilibrium 
eq(1) = card_eq(1);

%for each column (number of cards played)
for c=0:N-1

  %see which states have that number of cards
  ind = find(state_cards==c);
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% I think this is useless code.  The normalization will always be 1.
% Don't know why this was in there in the first place.

%   %normalize from the previous columns' equilibrium weighted by the reshuffle probability
%   if(c>0)
%     eq(ind) = card_eq(c+1)*eq(ind)/((1-resh_trans_pr(c))*card_eq(c));
%   end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
  %for each one of the states in that column
  for c2=1:length(ind)  
    
    % find which states it can transition to, and add in it's
    % weighted contribution to the equilibrium of that target state.
    % This section directly calculates the terms of equation 19 in the paper
    
    cur = ind(c2);
    
    [L,M,H]=ind2sub([Ltot+1, Mtot+1, Htot+1], cur);
    L=L-1;
    M=M-1;
    H=H-1;
    
    R = N-(L+M+H);   %total remaining cards
    
    %check each row in the P matrix
    %for that state, if there are still low cards left, calculate
    %the transition to (L+1,M,H)
    if (L<Ltot)
      Lprob = (Ltot-L)/R;
      Pind = sub2ind([Ltot+1, Mtot+1, Htot+1], L+2, M+1, H+1);
      eq(Pind) = eq(Pind)+eq(cur)*Lprob*(1-resh_trans_pr(c+1));
    end
    
    %and the transition to (L,M+1,H)
    if (M<Mtot)
      Mprob = (Mtot-M)/R;
      Pind = sub2ind([Ltot+1, Mtot+1, Htot+1], L+1, M+2, H+1);
      eq(Pind) = eq(Pind)+eq(cur)*Mprob*(1-resh_trans_pr(c+1));
    end
  
    %and the transition to (L,M,H+1)
    if (H<Htot)
      Hprob = (Htot-H)/R;
      Pind = sub2ind([Ltot+1, Mtot+1, Htot+1], L+1, M+1, H+2);
      eq(Pind) = eq(Pind)+eq(cur)*Hprob*(1-resh_trans_pr(c+1));
    end
    
  end
  
  
end


% If desired, save a .mat file with the results

% Name of the file is eq<num decks>.mat where <num decks> is the number of
% decks in the shoe (fractions represented such that 1/4 is given the
% string 0_25).

% File contains vectors of length numstates which contain:
%
% low -  the number of low cards played in each state
% med -  the number of medium cards played in each state
% high - the number of high cards played in each state
% rem -  the number of cards remaining in the shoe in each state
% HLI -  the HLI number in each state
% eq -   the equilibrium of each state

if(writeout==1)
  str = num2str(D);
  ind = find(str=='.');
  str(ind) = '_';
  name = ['eq' str];
  
  save(name, 'low', 'med', 'high', 'rem', 'HLI', 'eq');
end

if(writeout==2)
  str = num2str(D);
  ind = find(str=='.');
  str(ind) = '_';
  name = ['eq' str];
  
  save(name, 'eq');
end
