function [RESH_TRAN_PR, CARD_EQ, P_RESH]=reshuffle(D, MEAN_RESH)
%RESHUFFLE Calculate reshuffle probabilities.
%   [RESH_TRAN_PR]=reshuffle(D, MEAN_RESH) calculates the reshuffle
%   transition probabilites for a shoe with D decks of cards (multiples of
%   1/4) and an expected reshuffle point at the fraction MEAN_RESH (.5 <
%   MEAN_RESH < 1).  RESHUFFLE distributes the reshuffle point over the
%   second half of the shoe according to a normalized Laplcian distribution
%   with a variance that is proportional to the size of the shoe.
%   RESH_TRAN_PR is then a 52*D length vector where RESH_TRAN_PR(n)
%   represents the probability that if we have successfully played n-1
%   cards, we will reshuffle after the nth card.
%  
%   [RESH_TRAN_PR, CARD_EQ, P_RESH]=reshuffle(D, MEAN_RESH) returns a
%   vector of equilibrium probabilites CARD_EQ, representing the fraction
%   of time we see each number of cards having been played.  The matrix
%   P_RESH has 52*D rows and columns, and contains the transition
%   probabilites when the number of cards played are considered as the
%   state space of a Markov chain.
%
%   The default settings are D=1 and MEAN_RESH=.75.
%
%   RESHUFFLE, without any output arguments, plots the distribution of the
%   reshuffle point, the reshuffle transition probabilites and the
%   equilibrium over the number of cards played.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Blackjack Markov Chain Toolbox, v1.0 (08 Jan 2009)
%%% 
%%% Companion to manuscript 
%%%    "A Markov Chain Analysis of Blackjack Strategy"
%%%    by Michael B. Wakin and Christopher J. Rozell, 2004.
%%%    available at: http://inside.mines.edu/~mwakin/papers/mcbj.pdf
%%%
%%% Software Copyright 2009 by 
%%%    Michael B. Wakin, Colorado School of Mines, mwakin@mines.edu
%%%    Christopher J. Rozell, Georgia Tech, crozell@gatech.edu 
%%%
%%% This software is released under a Creative Commons license
%%% (Attribution 3.0 Unported): 
%%%     http://creativecommons.org/licenses/by/3.0/
%%% You are free:
%%%    * to Share  to copy, distribute and transmit the work
%%%    * to Remix  to adapt the work
%%% Under the following conditions:
%%%   * You must attribute the work by retaining this header information
%%%     crediting the original authors and linking to the original
%%%     manuscript.
%%%   * For any reuse or distribution, you must make clear to others the 
%%%     license terms of this work.
%%%   * Any of the above conditions can be waived if you get permission 
%%%     from the copyright holder.
%%%   * Nothing in this license impairs or restricts the author's 
%%%     moral rights.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%set the expected fraction of the shoe to play before reshuffle 
if(nargin<2)
  resh_frac=.75;  
else
  resh_frac=MEAN_RESH;
end

%we assume that a reshuffle can only occur in the second half of the shoe
if(resh_frac>=1 | resh_frac<=.5)
  error('Mean reshuffle fraction must be between .5 and 1!')
end

%the number of decks in the shoe must be a multiple of 1/4 (for integer
%numbers of cards)
if(mod(D,1/4)~=0)
  error('Decks can only be in multiples of 1/4');
end


cards = 1:52*D;         %vector of card numbers
N=max(cards);           %total number of cards


%calculate a vector of the probability distribution of the reshuffle point
%probability distribution is Laplacian, with a variance proportional to
%the number of cards in the shoe
pr_resh_loc = zeros(1,N);
sigma2=N/10;
pr_resh_loc(floor(N/2):end) = (1/sqrt(2*sigma2)).*exp(-abs((floor(N/2):N)-N* ...
						  resh_frac)./(sqrt(sigma2/2)));

pr_resh_loc = pr_resh_loc / sum(pr_resh_loc);


%compute the reshuffle transition probability vector.  If n cards have been
%played, what is the probability that we reshuffle on the next card.
%This is the same as the probability of the reshuffle location being n,
%conditioned on the fact that a reshuffle has not occured in the first
%n-1 cards already
RESH_TRAN_PR(1)=0;
for c=1:N
  RESH_TRAN_PR(c+1) = pr_resh_loc(c)/sum(pr_resh_loc(c:end));
end

%In terms of Markov Chains, we need a state space and a transition
%matrix.  Each state represents the card number.  On each transition from
%state n, we transition back to state 1 (i.e., reshuffle) with the reshuffle
%transition probability we calculated above (RESH_TRAN_PR(n)).  Otherwise,
%we transition to the next state (n-1) with the remaining probability.
%The matrix P_RESH computes the transitions between all of these states.

P_RESH=zeros(N+1);
P_RESH(:,1) = RESH_TRAN_PR';
for count=1:length(RESH_TRAN_PR)-1
  P_RESH(count, count+1) = 1-RESH_TRAN_PR(count);
end

%P_RESH has a form that looks like the following example:
%P_RESH = [RESH_TRAN_PR(1) (1-RESH_TRAN_PR(1)) 0                    0 ;
%          RESH_TRAN_PR(2) 0                    (1-RESH_TRAN_PR(2)) 0 ;
%          RESH_TRAN_PR(3) 0                    0                    (1-RESH_TRAN_PR(3));
%          RESH_TRAN_PR(4) 0                    0                    0
%         ];


%This state space is relatively small, so we can find the equilibrium directly
CARD_EQ = ones(1,N+1)*inv(eye(N+1)-P_RESH+ones(N+1));

if(nargout==0)
  figure
  subplot(311)
  plot(pr_resh_loc)
  ylabel('Prob[reshuffle=n]')
  grid
  
  subplot(312)
  plot(RESH_TRAN_PR)
  ylabel('\rho_n - transition probability')
  grid
  
  subplot(313)
  plot(CARD_EQ)
  ylabel('\mu_n - equilibrium')
  xlabel('n - card number')
  grid
end